# -*- coding: utf-8 -*-
"""
This is the initialization module of the BM configuration plugin. Here, the
frame of the plugin window is created and the master loop is runned.

Created on Tue Jan 31 15:53:37 2017

@author: ms, mel
"""

import argparse
from tkinter import Tk
from tkinter import messagebox
from bm_config import res
import ctypes

myappid = "BM_PLUGIN"
ctypes.windll.shell32.SetCurrentProcessExplicitAppUserModelID(myappid)


def main(config_file=None):
    """This function receives the PORT number and the ID of the plugin.
    The data is used to connect via thrift to AMS.
    The dependencies are also checked in order to ensure the plugin
    will run."""

    parser = argparse.ArgumentParser(description=res.DESCRIPTION)

    parser.add_argument("-p", "--port", help=res.PORT_HELP, required=False)
    parser.add_argument("-id", "--identification", help=res.ID_HELP, required=False)

    args = vars(parser.parse_args())
    print(args)
    print("id", args["identification"])

    if check_dependencies():
        import wx
        from bm_config.BM_Plugin import Frame

        app = wx.App(False)
        Frame(None, res.APP_TITLE, args, config_file)

        app.MainLoop()


def check_dependencies():

    try:
        import wx

        del wx
        import lxml

        del lxml
        import PIL

        del PIL
        import ams_service

        del ams_service
        import thrift

        del thrift
        import pygments_anyscript

        del pygments_anyscript
        import configobj

        del configobj

        return True

    except ModuleNotFoundError:

        top = Tk()
        top.withdraw()
        messagebox.showerror(res.WARN_TITLE, res.WARN_TEXT)

        return False


if __name__ == "__main__":

    main()
