"""
This module is used to define and create the 'Settings' tab inside the plugin.
This tab will be used to select the AnyBody workspace where the BM statements
will be defined with the desired value.

Author: Ioan-Matei Sarivan @ AnyBody Technology A/S
E-mail: ms@anybodytech.com

Cretaed on 7th of February, 15:16, 2017

"""

import wx
import wx.html
import pygments
import pkg_resources

from pygments.formatters import HtmlFormatter
from pygments_anyscript import AnyScriptLexer
from pygments_anyscript import AnyScriptStyle

from bm_config import image
from bm_config import res


class AddressTab(wx.Panel):
    def __init__(self, parent):
        """The Address tab is created by instantiating this class."""

        wx.Panel.__init__(self, parent)

        self._parent = parent

        self._got_address = False
        self._any_file_address = None

        self._text = None

        self._lexer = AnyScriptLexer()
        self._formatter = HtmlFormatter(style=AnyScriptStyle, full=True, noclasses=True)

        self._make_gui_elements()

    def _make_gui_elements(self):
        self.SetBackgroundColour("WHITE")

        # set image to tab
        address = pkg_resources.resource_filename("bm_config", "res/Images/Hello.png")
        img = image.Image(self, address, 500)
        imageCtrl = img.get_image_ctrl()
        self._imageCtrl = imageCtrl
        #

        # make gui elements and sizers
        do_text = wx.StaticText(self, label=res.L_DO_TEXT)
        self._address_box = wx.StaticBox(self, -1, res.L_ADDRESS)
        self._address_box.SetBackgroundColour((240, 240, 240, 255))
        BROWSE_button = wx.Button(self._address_box, label="Browse")
        OK_address_button = wx.Button(self._address_box, wx.ID_OK)
        self._address_text_box = wx.TextCtrl(self._address_box, size=(270, 23))
        Text = wx.StaticText(self._address_box, label=res.L_TEXT)

        browse_sizer = wx.BoxSizer(wx.HORIZONTAL)
        browse_sizer.Add(
            self._address_text_box,
            0,
            wx.LEFT | wx.ALIGN_CENTRE_HORIZONTAL | wx.ALIGN_CENTRE_VERTICAL | wx.BOTTOM,
            3,
        )
        browse_sizer.Add(
            BROWSE_button,
            0,
            wx.LEFT | wx.ALIGN_CENTRE_HORIZONTAL | wx.ALIGN_CENTRE_VERTICAL | wx.BOTTOM,
            3,
        )
        browse_sizer.Add(
            OK_address_button,
            0,
            wx.LEFT
            | wx.ALIGN_CENTRE_HORIZONTAL
            | wx.ALIGN_CENTRE_VERTICAL
            | wx.BOTTOM
            | wx.RIGHT,
            3,
        )

        address_box_sizer = wx.BoxSizer(wx.VERTICAL)
        address_box_sizer.Add(wx.StaticLine(self._address_box), 0, wx.TOP, 30)
        address_box_sizer.Add(
            Text, 0,  wx.LEFT | wx.EXPAND | wx.RIGHT, 4
        )
        address_box_sizer.Add(browse_sizer, 0, wx.LEFT , 3)

        self._address_box.SetSizer(address_box_sizer)

        address_box_sizer.Fit(self._address_box)
        main_sizer = wx.BoxSizer(wx.VERTICAL)
        main_sizer.Add(imageCtrl, 0, wx.ALL | wx.ALIGN_CENTRE_HORIZONTAL, 5)
        main_sizer.Add(do_text, 0, wx.ALL | wx.ALIGN_CENTRE_HORIZONTAL, 20)
        main_sizer.Add(self._address_box, 0, wx.ALIGN_CENTRE | wx.ALL, 3)
        self._line = wx.StaticLine(self)
        main_sizer.Add(self._line, 0, wx.TOP, 100)
        self.SetSizer(main_sizer)
        #

        # gui elements event handlings
        self.Bind(wx.EVT_BUTTON, self._on_address_ok, id=OK_address_button.GetId())
        self.Bind(wx.EVT_BUTTON, self._on_browse, id=BROWSE_button.GetId())
        #

        self._browse_sizer = browse_sizer
        self._main_sizer = main_sizer
        self._add_box = self._address_box
        self._text1 = do_text
        self._adr_box = self._address_box
        self._address_text_box
        self._text2 = Text

        self.Show()

    def _on_browse(self, event):
        dialog = wx.FileDialog(
            self, res["DIALOG"], wildcard=".any files (*.any)|*.any", style=wx.FD_OPEN
        )
        if dialog.ShowModal() == wx.ID_OK:
            path = str(dialog.GetPath())
            self._address_text_box.SetValue(path)
        dialog.Destroy()

    def _on_address_ok(self, event):
        self._any_file_address = self._address_text_box.GetValue()
        if self._any_file_address == "":
            self._warning_message(res.WARNING)

        else:

            self._parent.GetParent().GetParent().create_remaining_tabs()

    def _warning_message(self, parent, message, caption="Warning!"):

        dialog = wx.MessageDialog(parent, message, caption, wx.OK | wx.ICON_WARNING)
        dialog.ShowModal()
        dialog.Destroy()

    def get_any_file_address(self):

        return self._any_file_address

    def set_any_file_address(self, any_file):

        self._any_file_address = any_file

    def make_any_text(self, true):
        if not self._got_address:
            self._main_sizer.Detach(self._imageCtrl)
            self._imageCtrl.Hide()
            self._text1.Hide()
            self._line.Hide()
            self._main_sizer.Detach(self._text1)
            self._main_sizer.Detach(self._line)
            if true:
                self._main_sizer.Detach(self._address_box)
                self._address_box.Hide()

            self._text = wx.html.HtmlWindow(self, pos=(0, 30), size=(602, 310))

            self._main_sizer.Add(self._text, 1, wx.ALL | wx.EXPAND, 3)
            self._main_sizer.Layout()
            self._got_address = True
        with open(self._any_file_address, "r") as file:
            string = file.read()

            file.close()

        string = pygments.highlight(string, self._lexer, self._formatter)
        self._text.SetPage(string)

    def update_text_box(self):
        with open(self._any_file_address, "r") as file:
            string = file.read()
            file.close()

        string = pygments.highlight(string, self._lexer, self._formatter)

        if self._text is not None:
            self._text.SetPage(string)

    def got_address(self):
        return self._got_address
