"""
This module is used to parse and modify '.xml' files. The BM statements
together with their definitions are kept in '.xml' files.

Author: Ioan-Matei Sarivan @ AnyBody Technology A/S
E-mail: ms@anybodytech.com

Created on 7th of February, 15:16, 2017

"""

import shutil
import os
import datetime
import sys

sys.path.append(os.path.dirname(__file__))

from toml_decoder import load as toml_load


class BMStatement(object):
    def __init__(self):
        """stuct like class which will contain the paramaters of a BM
        statement"""

        self.tag = None
        self.name = None
        self.default = None
        self.valuetype = None
        self.description = None
        self.nicename = None
        self.numofopt = 0
        self.options = []
        self.value = "None"


class BMConstant(object):
    def __init__(self):
        """struct like class which will contain the parameters of a BM
        constant (option a BM statement can take)"""

        self.tag = None
        self.name = None
        self.default = None
        self.valuetype = None
        self.description = None
        self.nicename = None


class Statements(object):
    def __init__(self, address):
        """Collects all the BM statements and their parameters from the toml file
        file"""

        self._address = address
        self._statement = None
        self._constant = None
        self._bm_data = toml_load(self._address)
        # self._counter = 0

    def constants(self):
        """The BM constants are collected (options the statements can take)"""

        self._constant = []

        # self._counter = 0

        dictionary = {}

        for name, child in self._bm_data["constants"].items():
            bm_constant = BMConstant()
            bm_constant.name = name

            if "default" in child:
                bm_constant.default = child["default"]

            else:
                bm_constant.default = "None"

            if "descr" in child:
                bm_constant.description = child["descr"]

            else:
                bm_constant.description = "None"

            if "valuetype" in child:
                bm_constant.valuetype = child["valuetype"]

            else:
                bm_constant.valuetype = "None"

            if "nicename" in child:
                bm_constant.nicename = child["nicename"]
            else:
                bm_constant.nicename = "None"

            bm_constant.tag = "constant"

            self._constant.append(bm_constant)
            dictionary[bm_constant.name] = bm_constant

        return self._constant, dictionary

    def parameters(self):
        """The BM statements are collected"""
        self._statement = []
        dictionary = {}

        for name, child in self._bm_data["parameters"].items():
            bm_statement = BMStatement()
            bm_statement.name = name

            if "nicename" in child:
                bm_statement.nicename = child["nicename"]
            else:
                bm_statement.nicename = "None"

            if "default" in child:
                default_value = child["default"]
                if default_value in dictionary:
                    default_value = dictionary[default_value].default
                bm_statement.default = default_value
                if "BOTH" in bm_statement.default:
                    bm_statement.default = "both"
            else:
                bm_statement.default = "None"

            if "descr" in child:
                bm_statement.description = child["descr"]
            else:
                bm_statement.description = None

            if "valuetype" in child:
                bm_statement.valuetype = child["valuetype"]
            else:
                bm_statement.valuetype = None

            if "value" in child:
                bm_statement.value = child["value"]
            else:
                bm_statement.value = "None"

            bm_statement.tag = "parameter"

            if "opt" in child:
                bm_statement.numofopt = len(child["opt"])
            else:
                bm_statement.numofopt = 0

            if bm_statement.numofopt:
                for option in child["opt"]:
                    bm_statement.options.append(option)
                    if "SIMPLE" in option and bm_statement.default == "both":
                        bm_statement.default = option
            else:
                bm_statement.options.append(None)

            if "plugin_skip" not in child:
                self._statement.append(bm_statement)
                dictionary[bm_statement.name] = bm_statement

        # self._counter = 0
        return self._statement, dictionary
