"""
This module is used to create the 'Drivers' tab of the plugin.

Author: Ioan-Matei Sarivan @ AnyBody Technology A/S
E-mail: ms@anybodytech.com

Cretaed on 17th of January, 12:39, 2017

"""

import wx

from bm_config import images


class DriversTab(wx.Panel):
    def __init__(self, parent, args):
        """The Drivers tab is created by instantiating this class."""

        wx.Panel.__init__(self, parent)

        self._any_file = args.any_file
        self._statements = args.statements
        self._pr_dict = args.statements_dictionary
        self._options = args.options
        self._op_dict = args.options_dictionary
        self._memento = args.memento

        self._texts_ctrl = []
        self._checkboxes = []
        self._choices = []

        imgs = images.Images(self, "Other_Tabs", 500, 430)
        imgs.set_image("Body")
        self._make_gui_elements()

    def update_tab(self):
        for index in range(0, self.grid.GetNumberRows()):
            name = self.grid.GetCellValue(index, 0)
            for param in self._statements:
                if param.nicename == name:
                    value = self._any_file.Get_parameter(param.name)
                    test1 = "None" in value or "Multiple" in value
                    test2 = "DEFAULT" in param.name
                    test3 = "WEAK" in param.name
                    if test1:
                        value = "DEFAULT"
                    if test1 and (test2 or test3):
                        value = "ON"
                    self.grid.SetCellValue(index, 1, value)

        for chbox in self._checkboxes:
            op = self._any_file.Get_parameter(chbox.Name)
            if op == "None":
                default = self._pr_dict[chbox.Name].default
                if any(key in default for key in ["NONE", "OFF"]):
                    switch = False
                else:
                    switch = True
                chbox.SetValue(switch)
            else:
                if any(key in op for key in ["NONE", "OFF"]):
                    switch = False
                else:
                    switch = True
                chbox.SetValue(switch)

            if chbox.Name == "BM_GLOBAL_REFERENCE_FRAME_SWITCH":
                if chbox.GetValue():
                    self._texts_ctrl[0].Enable()
                else:
                    self._texts_ctrl[0].Disable()

        for text in self._texts_ctrl:
            value = self._any_file.Get_parameter(text.Name)
            if value != "None":
                text.ChangeValue(value)
            else:
                text.ChangeValue(self._pr_dict[text.Name].default)

    def _make_gui_elements(self):
        """The GUI elements inside the Drivers tab are created"""

        grid = wx.grid.Grid(
            self, -1, pos=(25, 55), size=(327, 200), name="MannequinDrivers"
        )
        grid.CreateGrid(1, 2)
        grid.SetColLabelValue(0, "Name")
        grid.SetColLabelValue(1, "Switch")
        grid.ShowScrollbars(wx.SHOW_SB_NEVER, wx.SHOW_SB_DEFAULT)
        grid.HideRowLabels()

        grid.SetColSize(0, 210)
        grid.SetColSize(1, 100)
        grid.DisableDragColSize()
        grid.DisableDragRowSize()

        grid_id = grid.GetId()
        default_op = ["ON", "OFF"]
        choice_op = ["DEFAULT", "ON", "OFF"]
        delete = True
        row = 0
        index = 0
        for parameter in self._statements:
            if "BM_MANNEQUIN_DRIVER" in parameter.name:
                if grid.GetNumberRows() and delete is True:
                    grid.DeleteRows(0, grid.GetNumberRows(), True)
                    delete = False
                grid.AppendRows()
                if (
                    "BM_MANNEQUIN_DRIVER_DEFAULT" in parameter.name
                    or "BM_MANNEQUIN_DRIVER_WEAK_SWITCH" in parameter.name
                ):
                    grid.SetCellBackgroundColour(0, 0, (200, 200, 255))
                    grid.SetCellBackgroundColour(0, 1, (200, 200, 255))
                    choice_editor = wx.grid.GridCellChoiceEditor(default_op, False)
                    grid.SetCellEditor(row, 1, choice_editor)
                # if parameter.nicename == 'None':
                #    print(parameter.name)
                else:
                    choice_editor = wx.grid.GridCellChoiceEditor(choice_op, False)
                    grid.SetCellEditor(row, 1, choice_editor)
                grid.SetCellValue(row, 0, parameter.nicename)

                adv_option = self._any_file.Get_parameter(parameter.name)
                if "None" in adv_option:
                    grid.SetCellValue(row, 1, "DEFAULT")
                elif "ON" in adv_option:
                    grid.SetCellValue(row, 1, "ON")
                elif "OFF":
                    grid.SetCellValue(row, 1, "OFF")

                row += 1
            index += 1

        self.Bind(wx.grid.EVT_GRID_CELL_CHANGED, self._on_cell_edit, id=grid_id)
        self.grid = grid

        line = wx.StaticLine(
            self, pos=(24, 55), style=wx.LI_VERTICAL, name="LeftLine", size=(2, 202)
        )
        line = wx.StaticLine(
            self, pos=(285, 55), style=wx.LI_VERTICAL, name="RightLine", size=(2, 202)
        )
        line = wx.StaticLine(self, pos=(24, 255), style=wx.LI_HORIZONTAL, size=(261, 2))
        line.SetName("BottomLine")
        name = "BM_GLOBAL_REFERENCE_FRAME"
        text_ctrl = wx.TextCtrl(self, pos=(170, 361), size=(150, 23), name=name)
        text_ctrl.SetValue(self._pr_dict[name].default)
        self.Bind(wx.EVT_TEXT, self._on_text_ctrl_2, id=text_ctrl.GetId())
        self._texts_ctrl.append(text_ctrl)

        name = "BM_GLOBAL_REFERENCE_FRAME_SWITCH"
        checkbox = wx.CheckBox(
            self, pos=(25, 365), name=name, label="Global reference frame:"
        )
        value = self._any_file.Get_value(self._pr_dict[name])
        if any(key in value for key in ["NONE", "OFF"]):
            switch = False
        else:
            switch = True
        checkbox.SetValue(switch)
        if checkbox.GetValue():
            self._texts_ctrl[0].Enable()
        else:
            self._texts_ctrl[0].Disable()
        self._checkboxes.append(checkbox)

        for index in range(0, len(self._checkboxes)):
            self.Bind(
                wx.EVT_CHECKBOX, self._on_check_box, id=self._checkboxes[index].GetId()
            )

    def _update_any(self):
        for chbox in self._checkboxes:
            for statement in self._statements:
                for chbox_name in chbox.Name.split():
                    if statement.name == chbox_name:
                        value = chbox.GetValue()
                        for option in statement.options:
                            if value:
                                if option is None:
                                    self._any_file.Set_parameter(statement, "ON")
                                    statement.value = True
                                elif "NONE" not in option or "OFF" not in option:
                                    self._any_file.Set_parameter(statement, option)
                                    statement.value = True
                            else:
                                if option is None:
                                    self._any_file.Set_parameter(statement, "OFF")
                                    statement.value = False
                                elif "NONE" in option or "OFF" in option:
                                    self._any_file.Set_parameter(statement, option)
                                    statement.value = False

        self._memento.create_memento(self.Parent.GetSelection())

    # Events: #####################################################################

    def _on_cell_edit(self, event):

        row = event.GetRow()
        column = event.GetCol()
        grid = self.grid
        cell_value = grid.GetCellValue(row, column)
        # if row == 0:
        #     for index in range(1, grid.GetNumberRows()):
        #         if 'Weak' not in grid.GetCellValue(index, 0):
        #             grid.SetCellValue(index, column, cell_value)

        #             par = grid.GetCellValue(index, column-1)
        #             for param in self._statements:
        #                 if param.nicename == par:
        #                     self._any_file.Set_parameter(param, cell_value)

        par = grid.GetCellValue(row, column - 1)
        for param in self._statements:
            if param.nicename == par:
                if "DEFAULT" in cell_value:
                    self._any_file.Remove_parameter(param)
                else:
                    self._any_file.Set_param(param.name, cell_value)

    def _on_check_box(self, event):
        self._update_any()
        if event.GetInt():
            self._texts_ctrl[0].Enable()
        else:
            self._texts_ctrl[0].Disable()

    def _on_text_ctrl_1(self, event):
        text = self._texts_ctrl[0]
        value = text.GetValue()
        self._any_file.Set_parameter(self._pr_dict[text.Name], value)

    def _on_text_ctrl_2(self, event):
        text = self._texts_ctrl[0]
        value = text.GetValue()
        self._any_file.Set_parameter(self._pr_dict[text.Name], value)
