# -*- coding: utf-8 -*-
"""
This module is used to colect the images needed for the graphics in the
plugin. All the needed images are stored in a list for each tab, from where
an image is chosen to be displayed in the tab using the name of the image. The
name of the image is generated from the values of the GUI elements in the tab.

Created on Thu Apr  6 16:01:04 2017

Author: Ioan-Matei Sarivan @ AnyBody Technology A/S
E-mail address: ms@anybodytech.com
"""

import pkg_resources
import wx

# from PIL import Image


class Images:
    def __init__(self, parent=None, folder="", max_dimension=None, position=None):
        """Collects all the images inside the passed folder making them ready
        to be displayed in the tab.

        parent -- self should be passed
        folder -- the folder where the images are located, usually the name of
        the tab
        max_dimenstion -- the maximum dimension of the image
        position -- the position of the image in the tab on the x axis
        """

        self.parent = parent
        self._position = position
        path = pkg_resources.resource_filename("bm_config", "/res/Images/" + folder)
        self._address = path
        self._max_dimension = max_dimension
        self._current_image = None

    def set_image(self, code):
        """Shoes the image with the given name on tab

        code -- the name of the image to be displayed on tab
        """
        address = self._address + "\\" + code + ".png"
        image = self._get_image(address, 0)
        imageCtrl = wx.StaticBitmap(self.parent, wx.ID_ANY, wx.Bitmap(image))
        imageCtrl.SetPosition((self._position, 1))
        imageCtrl.Show()
        if self._current_image is not None:
            self._current_image.Destroy()
        self._current_image = imageCtrl

    def _get_image(self, image_address, pretty_mode=0):
        """Configures the image in order to fit the size of the tab.

        image_address -- the absolute path of the image
        pretty_mode -- if 1 the image is resized in order to fit the plugin and
        it saves it. This is only necessary when new images are added.
        """

        if pretty_mode == 1:
            image = Image.open(image_address)

            width, height = image.size

            if width != self._max_dimension or height != self._max_dimension:
                if width > height:
                    height = self._max_dimension * height / width
                    width = self._max_dimension

                else:
                    width = self._max_dimension * width / height
                    height = self._max_dimension

            size = (int(width), int(height))
            image = image.resize(size, Image.ANTIALIAS)
            image.save(image_address)
            image = wx.Image(image_address, wx.BITMAP_TYPE_ANY)
            return image

        else:
            image = wx.Image(image_address, wx.BITMAP_TYPE_ANY)
            if (
                image.GetWidth() != self._max_dimension
                and image.GetHeight() != self._max_dimension
            ):

                width = image.GetWidth()
                height = image.GetHeight()
                if width > height:
                    height = self._max_dimension * height / width
                    width = self._max_dimension

                else:
                    width = self._max_dimension * width / height
                    height = self._max_dimension
                image = image.Scale(width, height)
                return image
            else:
                return wx.Image(image_address, wx.BITMAP_TYPE_ANY)
