"""
This module is used create the 'Legs' tab inside the plugin.

Author: Ioan-Matei Sarivan @ AnyBody Technology A/S
E-mail: ms@anybodytech.com

Cretaed on 7th of February, 15:16, 2017

"""

import wx

from bm_config import images


class LegsTab(wx.Panel):
    def __init__(self, parent, args):
        """The Legs tab is created by instantiating this class."""
        wx.Panel.__init__(self, parent)

        self._any_file = args.any_file
        self._statements = args.statements
        self._pr_dict = args.statements_dictionary
        self._options = args.options
        self._op_dict = args.options_dictionary
        self._memento = args.memento
        self._current_image = None

        self._choices = {}
        self._texts_ctrl = {}
        self._checkboxes = {}

        self._imgs = images.Images(self, "Legs_Tab", 500, 400)

        self._make_gui_elements()

    def update_tab(self):
        """Method for updating GUI elements following an external action."""
        for key in self._choices:
            choice = self._choices[key]
            name = choice.Name
            option = self._any_file.Get_parameter(name)
            if option == "None":
                choice.SetStringSelection(self._pr_dict[name].default)
            else:
                choice.SetStringSelection(self._op_dict[option].nicename)

        for key in self._checkboxes:
            chbox = self._checkboxes[key]
            option = self._any_file.Get_parameter(chbox.Name)
            if option == "None":
                default = self._pr_dict[chbox.Name].default
                if any(key in default for key in ["NONE", "OFF"]):
                    switch = False
                else:
                    switch = True
                chbox.SetValue(switch)
            else:
                if any(key in option for key in ["NONE", "OFF"]):
                    switch = False
                else:
                    switch = True
                chbox.SetValue(switch)

        for key in self._texts_ctrl:
            text = self._texts_ctrl[key]
            value = self._any_file.Get_parameter(text.Name)
            if value != "None":
                text.ChangeValue(value)
            else:
                text.ChangeValue(self._pr_dict[text.Name].default)

        self._set_image()

    def _make_gui_elements(self):
        self._make_choice("BM_LEG_MUSCLES_LEFT", ["BOTH", "OFF"], (160, 81))
        self._make_choice("BM_LEG_MUSCLES_RIGHT", ["BOTH", "OFF"], (160, 51))
        self._make_choice("BM_LEG_TRUNK_INTERFACE", ["NONE"], (160, 171))

        self._create_checkbox((25, 85), "BM_LEG_MUSCLES_LEFT")
        self._create_checkbox((25, 55), "BM_LEG_MUSCLES_RIGHT")
        self._create_checkbox((25, 175), "BM_LEG_TRUNK_INTERFACE")

        wx.StaticText(
            self, pos=(25, 115), name="BM_LEG_STRENGTH_INDEX", label="Strength index:"
        )
        wx.StaticText(self, pos=(25, 30), name="LEGS_TITLE", label="Muscles:")

        name = "BM_LEG_STRENGTH_INDEX"
        text_ctrl = wx.TextCtrl(self, pos=(160, 111), name=name)
        value = self._any_file.Get_value(self._pr_dict[name])
        text_ctrl.SetValue(value)
        self.Bind(wx.EVT_TEXT, self._on_text_ctrl, id=text_ctrl.GetId())
        self._texts_ctrl[name] = text_ctrl

    def _create_checkbox(self, pos, name):
        label = self._pr_dict[name].nicename
        chbox = wx.CheckBox(self, pos=pos, label=label, name=name, style=wx.CHK_3STATE)
        value = self._any_file.Get_value(self._pr_dict[name])
        if any(key in value for key in ["NONE", "OFF"]):
            switch = False
        else:
            switch = True
        chbox.SetValue(switch)
        self.Bind(wx.EVT_CHECKBOX, self._on_check_box, id=chbox.GetId())
        self._checkboxes[name] = chbox

    def _make_choice(self, name, exceptions, pos):
        choices = self._pr_dict[name].options
        choices = [x for x in choices if not any(y in x for y in exceptions)]
        choices = [self._op_dict[x].nicename for x in choices]
        choice = wx.Choice(self, pos=pos, choices=choices, name=name)
        value = self._any_file.Get_value(self._pr_dict[name])
        value = self._op_dict[value].nicename
        if "No" not in value:
            choice.SetStringSelection(value)
        else:
            choice.SetSelection(1)
        self.Bind(wx.EVT_CHOICE, self._on_choice, id=choice.GetId())
        self._choices[name] = choice

    def _update_any(self):
        for key in self._checkboxes:
            chbox = self._checkboxes[key]
            for statement in self._statements:
                for chbox_name in chbox.Name.split():
                    if statement.name == chbox_name:
                        value = chbox.GetValue()
                        for option in statement.options:
                            if value:
                                if option is None:
                                    self._any_file.Set_parameter(statement, "ON")
                                    statement.value = True
                                    break
                                elif "NONE" not in option and "OFF" not in option:
                                    self._any_file.Set_parameter(statement, option)
                                    statement.value = True
                                    break
                            else:
                                if option is None:
                                    self._any_file.Set_parameter(statement, "OFF")
                                    statement.value = False
                                    break
                                elif "NONE" in option or "OFF" in option:
                                    self._any_file.Set_parameter(statement, option)
                                    statement.value = False
                                    break

        for key in self._choices:
            choice = self._choices[key]
            for statement in self._statements:
                for choice_name in choice.Name.split():
                    if statement.name == choice_name:
                        value = choice.GetStringSelection()
                        for option in self._options:
                            if option.nicename == value and statement.value:
                                self._any_file.Set_parameter(statement, option.name)
                                break
                        break

        self._memento.create_memento(self.Parent.GetSelection())
        self.update_tab()

    def _set_image(self):
        self._checkboxes["BM_LEG_MUSCLES_LEFT"].Enable()
        self._checkboxes["BM_LEG_MUSCLES_RIGHT"].Enable()

        right_muscles = self._any_file.Get_parameter("BM_LEG_MUSCLES_RIGHT")
        left_muscles = self._any_file.Get_parameter("BM_LEG_MUSCLES_LEFT")

        if right_muscles in "None" or "SIMPLE" in right_muscles:
            right_muscles = "1"
        elif "HILL" in right_muscles:
            right_muscles = "2"
        else:
            right_muscles = "0"

        if left_muscles in "None" or "SIMPLE" in left_muscles:
            left_muscles = "1"
        elif "HILL" in left_muscles:
            left_muscles = "2"
        else:
            left_muscles = "0"

        right_leg = self._any_file.Get_parameter("BM_LEG_RIGHT")
        left_leg = self._any_file.Get_parameter("BM_LEG_LEFT")
        leg_type = self._any_file.Get_parameter("BM_LEG_MODEL")

        if "TLEM" in leg_type or leg_type in "None":
            right_type = "2"
            left_type = "2"
        else:
            right_type = "1"
            left_type = "1"

        if right_leg in "None":
            right_type = "2"
        elif "OFF" in right_leg:
            right_type = "0"
            right_muscles = "0"
            self._checkboxes["BM_LEG_MUSCLES_RIGHT"].Disable()

        if left_leg in "None":
            left_type = "2"
        elif "OFF" in left_leg:
            left_type = "0"
            left_muscles = "0"
            self._checkboxes["BM_LEG_MUSCLES_LEFT"].Disable()

        name = right_type + right_muscles + left_type + left_muscles

        if not self._checkboxes["BM_LEG_MUSCLES_LEFT"].GetValue():
            self._choices["BM_LEG_MUSCLES_LEFT"].Disable()
        else:
            self._choices["BM_LEG_MUSCLES_LEFT"].Enable()

        if not self._checkboxes["BM_LEG_MUSCLES_RIGHT"].GetValue():
            self._choices["BM_LEG_MUSCLES_RIGHT"].Disable()
        else:
            self._choices["BM_LEG_MUSCLES_RIGHT"].Enable()

        if not self._checkboxes["BM_LEG_TRUNK_INTERFACE"].GetValue():
            self._choices["BM_LEG_TRUNK_INTERFACE"].Disable()
        else:
            self._choices["BM_LEG_TRUNK_INTERFACE"].Enable()

        if not self._checkboxes["BM_LEG_MUSCLES_LEFT"].Enabled:
            self._choices["BM_LEG_MUSCLES_LEFT"].Disable()

        if not self._checkboxes["BM_LEG_MUSCLES_RIGHT"].Enabled:
            self._choices["BM_LEG_MUSCLES_RIGHT"].Disable()

        if self._current_image is None:
            self._current_image = True
        elif self._current_image != name:
            self._imgs.set_image(name)
            self._current_image = name

    # Events: #####################################################################

    def _on_check_box(self, event):
        self._update_any()

    def _on_choice(self, event):
        self._update_any()

    def _on_text_ctrl(self, event):
        text = self._texts_ctrl["BM_LEG_STRENGTH_INDEX"]
        value = text.GetValue()
        self._any_file.Set_parameter(self._pr_dict[text.Name], value)
        self._memento.create_memento(self.Parent.GetSelection())
