"""
This module is used to define and create the 'Trunk' tab inside the plugin.

Author: Ioan-Matei Sarivan @ AnyBody Technology A/S
E-mail: ms@anybodytech.com

Cretaed on 7th of February, 15:16, 2017

"""

import wx
from bm_config import images


class Cell:
    def __init__(self, row, column, name):
        self.row = row
        self.column = column
        self.name = name


class HeadCell:
    def __init__(self, row, column, name):
        self.row = row
        self.column = column
        self.name = name


class TrunkTab(wx.Panel):
    def __init__(self, parent, args):
        """The Trunk tab is created by instantiating this class."""

        wx.Panel.__init__(self, parent)

        self._any_file = args.any_file
        self._statements = args.statements
        self._pr_dict = args.statements_dictionary
        self._options = args.options
        self._op_dict = args.options_dictionary
        self._memento = args.memento
        self._current_image = None

        self._texts_ctrl = {}
        self._grids = []
        self._cells = {}
        self._head_cells = []

        self._imgs = images.Images(self, "Trunk_Tab", 370, 420)

        self._make_gui_elements()

    def update_tab(self):

        for cell in self._cells.values():
            op = self._any_file.Get_parameter(cell.name)
            if op != "None":
                op = self._op_dict[op].nicename
                self._grid.SetCellValue(cell.row, cell.column, op)
            else:
                op = self._pr_dict[cell.name].default
                op = self._op_dict[op].nicename
                self._grid.SetCellValue(cell.row, cell.column, op)

        for key in self._texts_ctrl:
            text = self._texts_ctrl[key]
            value = self._any_file.Get_parameter(text.Name)
            if value != "None":
                text.ChangeValue(value)
            else:
                text.ChangeValue(self._pr_dict[text.Name].default)

        self._set_image()

    def _make_gui_elements(self):
        """The GUI elements inside the Trunk tab are created"""

        grid = wx.grid.Grid(self, -1, pos=(5, 5), size=(415, 92), name="TrunkGrid")
        grid.CreateGrid(3, 4)
        grid.SetColLabelValue(0, "Muscles")
        grid.SetColLabelValue(1, "Disc Stiffness")
        grid.SetColLabelValue(2, "Rhythm")
        grid.SetColLabelValue(3, "Ligaments")

        grid.SetRowLabelValue(0, "Whole Trunk")
        grid.SetRowLabelValue(1, "Cervical")
        grid.SetRowLabelValue(2, "Lumbar")
        grid.SetColSize(0, 90)
        grid.SetColSize(1, 110)
        grid.SetColSize(2, 65)
        grid.SetColSize(3, 65)
        grid.DisableDragColSize()
        grid.DisableDragRowSize()
        grid.ShowScrollbars(wx.SHOW_SB_NEVER, wx.SHOW_SB_NEVER)

        self._grid = grid

        self._set_editor(0, 0, "BM_TRUNK_MUSCLES")
        cell = HeadCell(0, 0, "BM_TRUNK_MUSCLES")
        self._head_cells.append(cell)
        self._set_editor(0, 1, "BM_TRUNK_DISC_STIFFNESS")
        cell = HeadCell(0, 1, "BM_TRUNK_DISC_STIFFNESS")
        self._head_cells.append(cell)
        self._set_editor(0, 2, "BM_TRUNK_RHYTHM")
        cell = HeadCell(0, 2, "BM_TRUNK_RHYTHM")
        self._head_cells.append(cell)
        self._set_editor(0, 3, "BM_TRUNK_LIGAMENTS")
        cell = HeadCell(0, 3, "BM_TRUNK_LIGAMENTS")
        self._head_cells.append(cell)
        name = "BM_TRUNK_CERVICAL_MUSCLES"
        self._set_editor(1, 0, name)
        cell = Cell(1, 0, name)
        self._cells[name] = cell
        name = "BM_TRUNK_CERVICAL_DISC_STIFFNESS"
        self._set_editor(1, 1, name)
        cell = Cell(1, 1, name)
        self._cells[name] = cell
        name = "BM_TRUNK_CERVICAL_RHYTHM"
        self._set_editor(1, 2, name)
        cell = Cell(1, 2, name)
        self._cells[name] = cell
        # name = 'BM_TRUNK_CERVICAL_LIGAMENTS'
        # self._set_editor(1, 3, name)
        # cell = Cell(1, 3, name)
        # self._cells[name] = cell
        name = "BM_TRUNK_LUMBAR_MUSCLES"
        self._set_editor(2, 0, name)
        cell = Cell(2, 0, name)
        self._cells[name] = cell
        name = "BM_TRUNK_LUMBAR_DISC_STIFFNESS"
        self._set_editor(2, 1, name)
        cell = Cell(2, 1, name)
        self._cells[name] = cell
        name = "BM_TRUNK_LUMBAR_RHYTHM"
        self._set_editor(2, 2, name)
        cell = Cell(2, 2, name)
        self._cells[name] = cell
        name = "BM_TRUNK_LUMBAR_LIGAMENTS"
        self._set_editor(2, 3, name)
        cell = Cell(2, 3, name)
        self._cells[name] = cell

        grid_id = grid.GetId()
        self.Bind(wx.grid.EVT_GRID_CELL_CHANGED, self._on_cell_edit, id=grid_id)
        self._grids.append(grid)

        text_ctrl = wx.TextCtrl(self, pos=(160, 131), name="BM_TRUNK_STRENGTH_INDEX")
        self.Bind(wx.EVT_TEXT, self._on_text_ctrl_0, id=text_ctrl.GetId())
        self._texts_ctrl["BM_TRUNK_STRENGTH_INDEX"] = text_ctrl

        text_ctrl = wx.TextCtrl(self, pos=(160, 161), name="BM_TRUNK_DATASET")
        self.Bind(wx.EVT_TEXT, self._on_text_ctrl_1, id=text_ctrl.GetId())
        self._texts_ctrl["BM_TRUNK_DATASET"] = text_ctrl

        wx.StaticText(
            self,
            pos=(25, 135),
            name="BM_TRUNK_STRENGTH_INDEX",
            label="Strength index: ",
        )
        wx.StaticText(
            self, pos=(25, 165), name="BM_TRUNK_DATASET", label="Trunk dataset: "
        )

    def _set_editor(self, row, column, name):
        choices = self._pr_dict[name].options
        if None in choices:
            choices = ["On", "Off"]
        else:
            choices = [
                x
                for x in choices
                if "BM_TRUNK_MUSCLES" not in x
                and "BM_TRUNK_DISC_STIFFNESS" not in x
                and "BM_TRUNK_RHYTHM" not in x
                and "BM_TRUNK_LIGAMENTS" not in x
            ]
            choices = [self._op_dict[x].nicename for x in choices]
        choice_editor = wx.grid.GridCellChoiceEditor(choices, False)
        self._grid.SetCellEditor(row, column, choice_editor)
        value = self._any_file.Get_value(self._pr_dict[name])
        value = self._op_dict[value].nicename
        self._grid.SetCellValue(row, column, value)

    def _update_any(self, row=1, column=None):

        if row:
            for cell in self._cells.values():
                value = self._grid.GetCellValue(cell.row, cell.column)
                for option in self._options:
                    if option.nicename == value:
                        for statement in self._statements:
                            if statement.name == cell.name:
                                op = option.name
                                self._any_file.Set_parameter(statement, op)
                                break
        else:
            hvalue = self._grid.GetCellValue(row, column)
            self._grid.SetCellValue(1, column, hvalue)
            self._grid.SetCellValue(2, column, hvalue)
            for cell in self._cells.values():
                value = self._grid.GetCellValue(cell.row, cell.column)
                for option in self._options:
                    if option.nicename == value:
                        for statement in self._statements:
                            if statement.name == cell.name:
                                op = option.name
                                self._any_file.Set_parameter(statement, op)
                                break

        self._memento.create_memento(self.Parent.GetSelection())
        self._grid.Refresh()
        self._set_image()

    def _get_code(self, name):
        cell = self._cells[name]
        value = self._grid.GetCellValue(cell.row, cell.column)
        if "On" in value:
            return 1
        elif "Off" in value:
            return 0
        else:
            for option in self._op_dict.values():
                if option.nicename == value:
                    return option.default
            else:
                return 1

    def _set_image(self, row=1, column=None):
        """Updates the image of the body model in the tab by loading the image
        with the name corresponding to the current settings in the tab. The
        method is used to update the values of the GUI elements as well, if
        values from other tabs or from the advanced tab grid were changed.
        """

        # value = self._any_file.Get_parameter('BM_ARM_RIGHT')
        # if value not in 'None':
        #     if 'OFF' in value:
        #         right_type = str(0)
        #     elif 'DETAILED' in value:
        #         right_type = str(2)
        #     else:
        #         right_type = str(1)
        # else:
        #     right_type = str(1)

        # value = self._any_file.Get_parameter('BM_ARM_LEFT')
        # if value not in 'None':
        #     if 'OFF' in value:
        #         left_type = str(0)
        #     elif 'DETAILED' in value:
        #         left_type = str(2)
        #     else:
        #         left_type = str(1)
        # else:
        #     left_type = str(1)

        # name = right_type + left_type

        # name = name + '1'

        name = str(self._get_code("BM_TRUNK_LUMBAR_MUSCLES"))
        name = name + str(self._get_code("BM_TRUNK_CERVICAL_MUSCLES"))
        name = name + str(self._get_code("BM_TRUNK_LUMBAR_LIGAMENTS"))

        if self._current_image is None:
            self._current_image = True
        elif self._current_image != name:
            self._imgs.set_image(name)
            self._current_image = name
        self._grids[0].Raise()
        self.Refresh()

    # Events: #####################################################################

    def _on_cell_edit(self, event):
        self._update_any(row=event.GetRow(), column=event.GetCol())

    def _on_text_ctrl_0(self, event):
        text = self._texts_ctrl["BM_TRUNK_STRENGTH_INDEX"]
        value = text.GetValue()
        self._any_file.Set_parameter(self._pr_dict[text.Name], value)
        self._memento.create_memento(self.Parent.GetSelection())

    def _on_text_ctrl_1(self, event):
        text = self._texts_ctrl["BM_TRUNK_DATASET"]
        value = text.GetValue()
        self._any_file.Set_parameter(self._pr_dict[text.Name], value)
        self._memento.create_memento(self.Parent.GetSelection())
